package net.chipkillmar.poker;

/**
 * A playing card, comprising a value ("8") and a suit ("Hearts").
 *
 * @author Chip Killmar
 */
public class Card {
    private final Value value;
    private final Suit suit;

    public Card(Value value, Suit suit) {
        if (value == null) {
            throw new NullPointerException("value");
        }
        if (suit == null) {
            throw new NullPointerException("suit");
        }

        this.value = value;
        this.suit = suit;
    }

    public Suit getSuit() {
        return suit;
    }

    public Value getValue() {
        return value;
    }

    @Override
    public String toString() {
        final StringBuilder sb = new StringBuilder();
        sb.append("Card");
        sb.append("{suit=").append(suit);
        sb.append(", value=").append(value);
        sb.append('}');
        return sb.toString();
    }

    @Override
    public boolean equals(Object object) {
        if (this == object) {
            return true;
        }
        if (object == null || getClass() != object.getClass()) {
            return false;
        }

        Card card = (Card) object;

        if (suit != card.suit) {
            return false;
        }
        if (value != card.value) {
            return false;
        }

        return true;
    }

    @Override
    public int hashCode() {
        int result = value.hashCode();
        result = 31 * result + suit.hashCode();
        return result;
    }
}