package net.chipkillmar.poker;

import java.util.Arrays;
import java.util.List;

/**
 * A main class, which right now serves as a test harness for testing the poker hand ranking logic.
 *
 * @author Chip Killmar
 */
public class Main {
    public static void main(String[] args) {
        testCheckValid();
        testRoyalFlush();
        testStraightFlush();
        testFourOfAKind();
        testFullHouse();
        testFlush();
        testStraight();
        testThreeOfAKind();
        testTwoPair();
        testOnePair();
        testHighCard();
        testWinningHand1();
        testWinningHand2();
        testWinningHand3();
        testWinningHand4();
    }

    private static void testRoyalFlush() {
        Card[] hand = new Card[5];
        hand[0] = new Card(Value.ACE, Suit.CLUBS);
        hand[1] = new Card(Value.KING, Suit.CLUBS);
        hand[2] = new Card(Value.QUEEN, Suit.CLUBS);
        hand[3] = new Card(Value.JACK, Suit.CLUBS);
        hand[4] = new Card(Value.TEN, Suit.CLUBS);
        assertEquals(Logic.rank(hand), new Tuple2<HandRanking, List<Value>>(HandRanking.STRAIGHT_FLUSH, Arrays.asList(Value.ACE)));
    }

    private static void testStraightFlush() {
        Card[] hand = new Card[5];
        hand[0] = new Card(Value.KING, Suit.CLUBS);
        hand[1] = new Card(Value.QUEEN, Suit.CLUBS);
        hand[2] = new Card(Value.JACK, Suit.CLUBS);
        hand[3] = new Card(Value.TEN, Suit.CLUBS);
        hand[4] = new Card(Value.NINE, Suit.CLUBS);
        assertEquals(Logic.rank(hand), new Tuple2<HandRanking, List<Value>>(HandRanking.STRAIGHT_FLUSH, Arrays.asList(Value.KING)));
    }

    private static void testFourOfAKind() {
        Card[] hand = new Card[5];
        hand[0] = new Card(Value.KING, Suit.CLUBS);
        hand[1] = new Card(Value.KING, Suit.DIAMONDS);
        hand[2] = new Card(Value.KING, Suit.HEARTS);
        hand[3] = new Card(Value.KING, Suit.SPADES);
        hand[4] = new Card(Value.NINE, Suit.CLUBS);
        assertEquals(Logic.rank(hand), new Tuple2<HandRanking, List<Value>>(HandRanking.FOUR_OF_A_KIND, Arrays.asList(Value.KING, Value.NINE)));

        hand[0] = new Card(Value.ACE, Suit.CLUBS);
        hand[1] = new Card(Value.KING, Suit.DIAMONDS);
        hand[2] = new Card(Value.KING, Suit.HEARTS);
        hand[3] = new Card(Value.KING, Suit.SPADES);
        hand[4] = new Card(Value.KING, Suit.CLUBS);
        assertEquals(Logic.rank(hand), new Tuple2<HandRanking, List<Value>>(HandRanking.FOUR_OF_A_KIND, Arrays.asList(Value.KING, Value.ACE)));
    }

    private static void testFullHouse() {
        Card[] hand = new Card[5];
        hand[0] = new Card(Value.KING, Suit.CLUBS);
        hand[1] = new Card(Value.KING, Suit.DIAMONDS);
        hand[2] = new Card(Value.KING, Suit.HEARTS);
        hand[3] = new Card(Value.NINE, Suit.SPADES);
        hand[4] = new Card(Value.NINE, Suit.CLUBS);
        assertEquals(Logic.rank(hand), new Tuple2<HandRanking, List<Value>>(HandRanking.FULL_HOUSE, Arrays.asList(Value.KING, Value.NINE)));

        hand[0] = new Card(Value.NINE, Suit.DIAMONDS);
        hand[1] = new Card(Value.KING, Suit.DIAMONDS);
        hand[2] = new Card(Value.KING, Suit.HEARTS);
        hand[3] = new Card(Value.NINE, Suit.SPADES);
        hand[4] = new Card(Value.NINE, Suit.CLUBS);
        assertEquals(Logic.rank(hand), new Tuple2<HandRanking, List<Value>>(HandRanking.FULL_HOUSE, Arrays.asList(Value.NINE, Value.KING)));
    }

    private static void testFlush() {
        Card[] hand = new Card[5];
        hand[0] = new Card(Value.KING, Suit.CLUBS);
        hand[1] = new Card(Value.QUEEN, Suit.CLUBS);
        hand[2] = new Card(Value.JACK, Suit.CLUBS);
        hand[3] = new Card(Value.TEN, Suit.CLUBS);
        hand[4] = new Card(Value.EIGHT, Suit.CLUBS);
        assertEquals(Logic.rank(hand), new Tuple2<HandRanking, List<Value>>(HandRanking.FLUSH, Arrays.asList(Value.KING, Value.QUEEN, Value.JACK, Value.TEN, Value.EIGHT)));
    }

    private static void testStraight() {
        Card[] hand = new Card[5];
        hand[0] = new Card(Value.KING, Suit.CLUBS);
        hand[1] = new Card(Value.QUEEN, Suit.DIAMONDS);
        hand[2] = new Card(Value.JACK, Suit.CLUBS);
        hand[3] = new Card(Value.TEN, Suit.CLUBS);
        hand[4] = new Card(Value.NINE, Suit.CLUBS);
        assertEquals(Logic.rank(hand), new Tuple2<HandRanking, List<Value>>(HandRanking.STRAIGHT, Arrays.asList(Value.KING)));

        hand[0] = new Card(Value.ACE, Suit.CLUBS);
        hand[1] = new Card(Value.TWO, Suit.DIAMONDS);
        hand[2] = new Card(Value.THREE, Suit.CLUBS);
        hand[3] = new Card(Value.FOUR, Suit.CLUBS);
        hand[4] = new Card(Value.FIVE, Suit.CLUBS);
        assertEquals(Logic.rank(hand), new Tuple2<HandRanking, List<Value>>(HandRanking.STRAIGHT, Arrays.asList(Value.FIVE)));
    }

    private static void testThreeOfAKind() {
        Card[] hand = new Card[5];
        hand[0] = new Card(Value.KING, Suit.CLUBS);
        hand[1] = new Card(Value.KING, Suit.DIAMONDS);
        hand[2] = new Card(Value.KING, Suit.HEARTS);
        hand[3] = new Card(Value.QUEEN, Suit.SPADES);
        hand[4] = new Card(Value.NINE, Suit.CLUBS);
        assertEquals(Logic.rank(hand), new Tuple2<HandRanking, List<Value>>(HandRanking.THREE_OF_A_KIND, Arrays.asList(Value.KING, Value.QUEEN, Value.NINE)));

        hand[0] = new Card(Value.QUEEN, Suit.CLUBS);
        hand[1] = new Card(Value.QUEEN, Suit.DIAMONDS);
        hand[2] = new Card(Value.QUEEN, Suit.HEARTS);
        hand[3] = new Card(Value.KING, Suit.SPADES);
        hand[4] = new Card(Value.NINE, Suit.CLUBS);
        assertEquals(Logic.rank(hand), new Tuple2<HandRanking, List<Value>>(HandRanking.THREE_OF_A_KIND, Arrays.asList(Value.QUEEN, Value.KING, Value.NINE)));

        hand[0] = new Card(Value.NINE, Suit.CLUBS);
        hand[1] = new Card(Value.NINE, Suit.DIAMONDS);
        hand[2] = new Card(Value.NINE, Suit.HEARTS);
        hand[3] = new Card(Value.KING, Suit.SPADES);
        hand[4] = new Card(Value.QUEEN, Suit.CLUBS);
        assertEquals(Logic.rank(hand), new Tuple2<HandRanking, List<Value>>(HandRanking.THREE_OF_A_KIND, Arrays.asList(Value.NINE, Value.KING, Value.QUEEN)));
    }

    private static void testTwoPair() {
        Card[] hand = new Card[5];
        hand[0] = new Card(Value.KING, Suit.CLUBS);
        hand[1] = new Card(Value.KING, Suit.DIAMONDS);
        hand[2] = new Card(Value.TWO, Suit.HEARTS);
        hand[3] = new Card(Value.NINE, Suit.SPADES);
        hand[4] = new Card(Value.NINE, Suit.CLUBS);
        assertEquals(Logic.rank(hand), new Tuple2<HandRanking, List<Value>>(HandRanking.TWO_PAIR, Arrays.asList(Value.KING, Value.NINE, Value.TWO)));

        hand[0] = new Card(Value.KING, Suit.CLUBS);
        hand[1] = new Card(Value.KING, Suit.DIAMONDS);
        hand[2] = new Card(Value.TWO, Suit.HEARTS);
        hand[3] = new Card(Value.TWO, Suit.SPADES);
        hand[4] = new Card(Value.NINE, Suit.CLUBS);
        assertEquals(Logic.rank(hand), new Tuple2<HandRanking, List<Value>>(HandRanking.TWO_PAIR, Arrays.asList(Value.KING, Value.TWO, Value.NINE)));

        hand[0] = new Card(Value.KING, Suit.CLUBS);
        hand[1] = new Card(Value.NINE, Suit.DIAMONDS);
        hand[2] = new Card(Value.TWO, Suit.HEARTS);
        hand[3] = new Card(Value.TWO, Suit.SPADES);
        hand[4] = new Card(Value.NINE, Suit.CLUBS);
        assertEquals(Logic.rank(hand), new Tuple2<HandRanking, List<Value>>(HandRanking.TWO_PAIR, Arrays.asList(Value.NINE, Value.TWO, Value.KING)));
    }

    private static void testOnePair() {
        Card[] hand = new Card[5];
        hand[0] = new Card(Value.KING, Suit.CLUBS);
        hand[1] = new Card(Value.KING, Suit.DIAMONDS);
        hand[2] = new Card(Value.TWO, Suit.HEARTS);
        hand[3] = new Card(Value.NINE, Suit.SPADES);
        hand[4] = new Card(Value.EIGHT, Suit.CLUBS);
        assertEquals(Logic.rank(hand), new Tuple2<HandRanking, List<Value>>(HandRanking.ONE_PAIR, Arrays.asList(Value.KING, Value.NINE, Value.EIGHT, Value.TWO)));

        hand[0] = new Card(Value.TWO, Suit.CLUBS);
        hand[1] = new Card(Value.TWO, Suit.DIAMONDS);
        hand[2] = new Card(Value.KING, Suit.HEARTS);
        hand[3] = new Card(Value.NINE, Suit.SPADES);
        hand[4] = new Card(Value.EIGHT, Suit.CLUBS);
        assertEquals(Logic.rank(hand), new Tuple2<HandRanking, List<Value>>(HandRanking.ONE_PAIR, Arrays.asList(Value.TWO, Value.KING, Value.NINE, Value.EIGHT)));

        hand[0] = new Card(Value.TWO, Suit.CLUBS);
        hand[1] = new Card(Value.EIGHT, Suit.DIAMONDS);
        hand[2] = new Card(Value.EIGHT, Suit.HEARTS);
        hand[3] = new Card(Value.KING, Suit.SPADES);
        hand[4] = new Card(Value.NINE, Suit.CLUBS);
        assertEquals(Logic.rank(hand), new Tuple2<HandRanking, List<Value>>(HandRanking.ONE_PAIR, Arrays.asList(Value.EIGHT, Value.KING, Value.NINE, Value.TWO)));

        hand[0] = new Card(Value.TWO, Suit.CLUBS);
        hand[1] = new Card(Value.NINE, Suit.DIAMONDS);
        hand[2] = new Card(Value.NINE, Suit.HEARTS);
        hand[3] = new Card(Value.KING, Suit.SPADES);
        hand[4] = new Card(Value.EIGHT, Suit.CLUBS);
        assertEquals(Logic.rank(hand), new Tuple2<HandRanking, List<Value>>(HandRanking.ONE_PAIR, Arrays.asList(Value.NINE, Value.KING, Value.EIGHT, Value.TWO)));
    }

    private static void testHighCard() {
        Card[] hand = new Card[5];
        hand[0] = new Card(Value.KING, Suit.CLUBS);
        hand[1] = new Card(Value.QUEEN, Suit.DIAMONDS);
        hand[2] = new Card(Value.TWO, Suit.HEARTS);
        hand[3] = new Card(Value.NINE, Suit.SPADES);
        hand[4] = new Card(Value.EIGHT, Suit.CLUBS);
        assertEquals(Logic.rank(hand), new Tuple2<HandRanking, List<Value>>(HandRanking.HIGH_CARD, Arrays.asList(Value.KING, Value.QUEEN, Value.NINE, Value.EIGHT, Value.TWO)));
    }

    private static void testWinningHand1() {
        Card[] hand1 = new Card[5];
        hand1[0] = new Card(Value.KING, Suit.CLUBS);
        hand1[1] = new Card(Value.QUEEN, Suit.DIAMONDS);
        hand1[2] = new Card(Value.TWO, Suit.HEARTS);
        hand1[3] = new Card(Value.NINE, Suit.SPADES);
        hand1[4] = new Card(Value.EIGHT, Suit.CLUBS);

        Card[] hand2 = new Card[5];
        hand2[0] = new Card(Value.KING, Suit.CLUBS);
        hand2[1] = new Card(Value.KING, Suit.DIAMONDS);
        hand2[2] = new Card(Value.TWO, Suit.HEARTS);
        hand2[3] = new Card(Value.NINE, Suit.SPADES);
        hand2[4] = new Card(Value.EIGHT, Suit.CLUBS);

        assertEquals(-1, Logic.determineWinner(hand1, hand2));
    }

    private static void testWinningHand2() {
        Card[] hand1 = new Card[5];
        hand1[0] = new Card(Value.KING, Suit.CLUBS);
        hand1[1] = new Card(Value.QUEEN, Suit.DIAMONDS);
        hand1[2] = new Card(Value.TWO, Suit.HEARTS);
        hand1[3] = new Card(Value.NINE, Suit.SPADES);
        hand1[4] = new Card(Value.EIGHT, Suit.CLUBS);

        Card[] hand2 = new Card[5];
        hand2[0] = new Card(Value.KING, Suit.SPADES);
        hand2[1] = new Card(Value.QUEEN, Suit.DIAMONDS);
        hand2[2] = new Card(Value.TWO, Suit.HEARTS);
        hand2[3] = new Card(Value.NINE, Suit.SPADES);
        hand2[4] = new Card(Value.EIGHT, Suit.CLUBS);

        assertEquals(0, Logic.determineWinner(hand1, hand2));
    }

    private static void testWinningHand3() {
        Card[] hand1 = new Card[5];
        hand1[0] = new Card(Value.ACE, Suit.CLUBS);
        hand1[1] = new Card(Value.QUEEN, Suit.DIAMONDS);
        hand1[2] = new Card(Value.TWO, Suit.HEARTS);
        hand1[3] = new Card(Value.NINE, Suit.SPADES);
        hand1[4] = new Card(Value.EIGHT, Suit.CLUBS);

        Card[] hand2 = new Card[5];
        hand2[0] = new Card(Value.ACE, Suit.SPADES);
        hand2[1] = new Card(Value.QUEEN, Suit.DIAMONDS);
        hand2[2] = new Card(Value.TWO, Suit.HEARTS);
        hand2[3] = new Card(Value.NINE, Suit.SPADES);
        hand2[4] = new Card(Value.SEVEN, Suit.CLUBS);

        assertEquals(1, Logic.determineWinner(hand1, hand2));
        assertEquals(-1, Logic.determineWinner(hand2, hand1));
    }

    private static void testWinningHand4() {
        Card[] hand1 = new Card[5];
        hand1[0] = new Card(Value.KING, Suit.CLUBS);
        hand1[1] = new Card(Value.QUEEN, Suit.CLUBS);
        hand1[2] = new Card(Value.JACK, Suit.CLUBS);
        hand1[3] = new Card(Value.TEN, Suit.CLUBS);
        hand1[4] = new Card(Value.EIGHT, Suit.CLUBS);

        Card[] hand2 = new Card[5];
        hand2[0] = new Card(Value.KING, Suit.CLUBS);
        hand2[1] = new Card(Value.QUEEN, Suit.CLUBS);
        hand2[2] = new Card(Value.JACK, Suit.CLUBS);
        hand2[3] = new Card(Value.TEN, Suit.CLUBS);
        hand2[4] = new Card(Value.SEVEN, Suit.CLUBS);

        assertEquals(1, Logic.determineWinner(hand1, hand2));
    }

    private static void testCheckValid() {
        Exception e = null;
        try {
            Logic.rank(null);
        } catch (Exception e2) {
            e = e2;
        }
        assertNotNull(e);

        e = null;
        try {
            Logic.rank(new Card[4]);
        } catch (Exception e2) {
            e = e2;
        }
        assertNotNull(e);

        e = null;
        try {
            Logic.rank(new Card[5]);
        } catch (Exception e2) {
            e = e2;
        }
        assertNotNull(e);

        e = null;
        Card[] hand = new Card[5];
        hand[0] = new Card(Value.ACE, Suit.CLUBS);
        hand[1] = new Card(Value.ACE, Suit.CLUBS);
        hand[2] = new Card(Value.TWO, Suit.HEARTS);
        hand[3] = new Card(Value.NINE, Suit.SPADES);
        hand[4] = new Card(Value.EIGHT, Suit.CLUBS);
        try {
            Logic.rank(hand);
        } catch (Exception e2) {
            e = e2;
        }
        assertNotNull(e);
    }

    private static void assertEquals(Object object1, Object object2) {
        if (object1 == null && object2 == null) {
            return;
        }
        if (object1 == null && object2 != null || object1 != null && object2 == null || !object1.equals(object2)) {
            throw new RuntimeException("Assertion failed, [object1=" + object1 + ", object2=" + object2);
        }
    }

    private static void assertNotNull(Object object1) {
        if (object1 == null) {
            throw new RuntimeException("Assertion failed, object1 is null");
        }
    }
}