package net.chipkillmar.poker;

/**
 * A generic 2-tuple class.
 *
 * @author Chip Killmar
 */
public class Tuple2<S, T> {
    private final S first;
    private final T second;

    public Tuple2(S first, T second) {
        this.first = first;
        this.second = second;
    }

    public S getFirst() {
        return first;
    }

    public T getSecond() {
        return second;
    }

    @Override
    public String toString() {
        final StringBuilder sb = new StringBuilder();
        sb.append("Tuple2");
        sb.append("{first=").append(first);
        sb.append(", second=").append(second);
        sb.append('}');
        return sb.toString();
    }

    @Override
    public boolean equals(Object object) {
        if (this == object) {
            return true;
        }
        if (object == null || getClass() != object.getClass()) {
            return false;
        }

        Tuple2 tuple2 = (Tuple2) object;
        if (first != null ? !first.equals(tuple2.first) : tuple2.first != null) {
            return false;
        }
        if (second != null ? !second.equals(tuple2.second) : tuple2.second != null) {
            return false;
        }

        return true;
    }

    @Override
    public int hashCode() {
        int result = first != null ? first.hashCode() : 0;
        result = 31 * result + (second != null ? second.hashCode() : 0);
        return result;
    }
}